using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline;
using Microsoft.Xna.Framework.Content.Pipeline.Graphics;
using Microsoft.Xna.Framework.Content.Pipeline.Processors;

using TInput = Microsoft.Xna.Framework.Content.Pipeline.Graphics.NodeContent;
using TOutput = System.Collections.Generic.Dictionary<string, MikuMikuDance.XNA.Motion.MotionData.MMDMotionData>;
using MikuMikuDance.XNA.Motion.MotionData;
using System.Diagnostics;

namespace MikuMikuDance.XNA.Motion
{
    /// <summary>
    /// ft̃[Vf[^f[^MMD[VƂXNAɃC|[g邽߂̃vZbT
    /// </summary>
    /// <remarks>MMDXłLoadModelMotionœǂݏo</remarks>
    [ContentProcessor(DisplayName = "f̃[VpvZbT : MikuMikuDance for XNA")]
    public class MMDModelMotionProcessor : ContentProcessor<TInput, TOutput>
    {

        /// <summary>
        /// Recϊ
        /// </summary>
        /// <param name="input">f[Vf[^</param>
        /// <param name="context">RegvZbT</param>
        /// <returns>MMD[Vf[^ for XNA</returns>
        public override TOutput Process(TInput input, ContentProcessorContext context)
        {
            ValidateMesh(input, context, null);

            // XPgT
            BoneContent skeleton = MeshHelper.FindSkeleton(input);

            if (skeleton == null)
                throw new InvalidContentException("XPg܂");

            // f̃bVꂼႤ[JWĂƈ
            // ʓ|Ȃ̂ŁASĂႤ(bV̕ϊW炩ߒ_
            // f[^ɓKp邱)
            FlattenTransforms(input, skeleton);

            // oChE|[YƃXPg\ǂݍ
            IList<BoneContent> bones = MeshHelper.FlattenSkeleton(skeleton);
            
            //bVAj[V̌
            TOutput result;

            result = ProcessAnimations(skeleton.Animations, bones);


            return result;
        }

        /// <summary>
        /// Aj[VMMDpɕϊ
        /// </summary>
        static Dictionary<string, MMDMotionData> ProcessAnimations(AnimationContentDictionary animations, IList<BoneContent> bones)
        {
            if (animations.Count == 0)
                throw new InvalidContentException("̃fɂ̓Aj[V񂪊܂܂Ă܂");
            // {[CfbNXɕϊ鎫e[u
            Dictionary<string, int> boneMap = new Dictionary<string, int>();
            for (int i = 0; i < bones.Count; i++)
            {
                string boneName = bones[i].Name;
                if (!string.IsNullOrEmpty(boneName))
                    boneMap.Add(boneName, i);
            }

            Dictionary<string, MMDMotionData> result = new Dictionary<string, MMDMotionData>();
            foreach (var animation in animations)
            {
                MMDMotionData motion = ProcessAnimation(animation.Value, boneMap, bones);
                result.Add(animation.Key, motion);
            }
            return result;
        }

        static MMDMotionData ProcessAnimation(AnimationContent animation, Dictionary<string, int> boneMap, IList<BoneContent> bones)
        {
            List<MMDBoneMotion> keyframes = new List<MMDBoneMotion>();

            //Aj[V`l
            foreach (var channel in animation.Channels)
            {
                if (!boneMap.ContainsKey(channel.Key))
                    continue;
                // L[t[̕ϊ
                foreach (AnimationKeyframe keyframe in channel.Value)
                {
                    //MMD̃[Vl*oCh|[Yf̃[VlȂ̂ŁAoCh|[Y̋t
                    Matrix InvBind = Matrix.Invert(bones[boneMap[channel.Key]].Transform);
                    Matrix transform = keyframe.Transform * InvBind;

                    // s̕
                    Quaternion rotation;
                    Vector3 translation;
                    Vector3 scale;
                    transform.Decompose(out scale, out rotation, out translation);
                    rotation.Normalize();

                    keyframes.Add(new MMDBoneMotion()
                    {
                        BoneName = channel.Key,
                        FrameNo = (uint)Math.Round((double)keyframe.Time.Ticks * 60.0 / (double)Stopwatch.Frequency),
                        Location = translation,
                        Quatanion = rotation,
                        Curve = CreateIdentityCurve()
                    });
                }
            }
            //\[g
            keyframes.Sort((x, y) => (int)((long)x.FrameNo - (long)y.FrameNo));
            //G[`FbN
            if (keyframes.Count == 0)
                throw new InvalidContentException(
                    "L[t[̖Aj[VłB");
            //MMDMotionDataɗ
            MMDMotionData result = new MMDMotionData();
            //L[t[Xgzɕϊ
            result.BoneMotions = keyframes.ToArray();
            //̑̂̂͂zɂ
            result.FaceMotions = new MMDFaceMotion[0];
            result.CameraMotions = new MMDCameraMotion[0];
            result.LightMotions = new MMDLightMotion[0];
            return result;

        }

        private static BezierCurve[] CreateIdentityCurve()
        {
            BezierCurve[] result = new BezierCurve[4];
            for (int i = 0; i < result.Length; i++)
            {
                result[i].v1 = new Vector2(0.25f, 0.25f);
                result[i].v2 = new Vector2(0.75f, 0.75f);
            }
            return result;
        }


        /// <summary>
        /// ̃bV̓XLAj[VɌ̂`FbN
        /// </summary>
        static void ValidateMesh(NodeContent node, ContentProcessorContext context,
                                 string parentBoneName)
        {
            MeshContent mesh = node as MeshContent;

            if (mesh != null)
            {
                // bV̐𒲂ׂ
                if (parentBoneName != null)
                {
                    context.Logger.LogWarning(null, null,
                        "{0}bV{1}{[̎qłB" +
                        "SkinnedModelProcessor̓bV{[̎qłP[X" +
                        "ΉĂ܂B",
                        mesh.Name, parentBoneName);
                }

                if (!MeshHasSkinning(mesh))
                {
                    context.Logger.LogWarning(null, null,
                        "{0}bV̓XLjO񂪂Ȃ̂ŕϊ܂",
                        mesh.Name);

                    mesh.Parent.Children.Remove(mesh);
                    return;
                }
            }
            else if (node is BoneContent)
            {
                // ̃m[h{[ȂA{[𒲍ł邱ƂoĂ
                parentBoneName = node.Name;
            }

            // ċAI(Ƀm[ĥŁA
            // qB̃Rs[𑖍)
            foreach (NodeContent child in new List<NodeContent>(node.Children))
                ValidateMesh(child, context, parentBoneName);
        }


        /// <summary>
        /// bVXLjOĂ邩ׂ
        /// </summary>
        static bool MeshHasSkinning(MeshContent mesh)
        {
            foreach (GeometryContent geometry in mesh.Geometry)
            {
                if (!geometry.Vertices.Channels.Contains(VertexChannelNames.Weights()))
                    return false;
            }

            return true;
        }


        /// <summary>
        /// SĂWԂɂȂ悤ɁAsKvȕϊs
        /// fEWIgɏĂt
        /// </summary>
        static void FlattenTransforms(NodeContent node, BoneContent skeleton)
        {
            foreach (NodeContent child in node.Children)
            {
                // XPg͏Ȃ
                if (child == skeleton)
                    continue;

                // [JϊsWIgɏĂ
                MeshHelper.TransformScene(child, child.Transform);

                // Ăt̂ŁA[JWϊs
                // Pʍs(Matrix.Identity)ɂȂ
                child.Transform = Matrix.Identity;

                // ċAĂяo
                FlattenTransforms(child, skeleton);
            }
        }

    }
}