﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework.Graphics;

namespace MikuMikuDanceXNADemo4
{
    /// <summary>
    /// フリップ可能なレンダリングターゲットクラス
    /// GPUとバッティングする可能性を排除するため
    /// </summary>
    class FlipRenderTarget2D : IDisposable
    {
        RenderTarget2D[] targets;
        int index;
        public RenderTarget2D RenderTareget { get { return targets[index]; } }

        /// <summary>
        /// レンダリングターゲットを生成
        /// </summary>
        /// <param name="graphicDevice">グラフィックデバイス</param>
        /// <param name="width">幅</param>
        /// <param name="height">高さ</param>
        /// <param name="numberLevels">ミップマップレベル</param>
        /// <param name="format">フォーマット</param>
        /// <param name="NumTarget">フリップ数</param>
        public void Setup(GraphicsDevice graphicDevice, int width, int height, int numberLevels,
            SurfaceFormat format, MultiSampleType multiSampleType, int multiSampleQuality, int NumTarget)
        {
            targets = new RenderTarget2D[NumTarget];
            for (int i = 0; i < NumTarget; i++)
            {
                targets[i] = new RenderTarget2D(graphicDevice, width, height, numberLevels, format, multiSampleType, multiSampleQuality);
            }
        }

        /// <summary>
        /// 使用するレンダリングターゲットを切り替え
        /// </summary>
        public void Flip()
        {
            if (++index >= targets.Length)
                index = 0;
        }

        /// <summary>
        /// リソースの破棄
        /// </summary>
        public void Dispose()
        {
            for (int i = 0; i < targets.Length; ++i)
            {
                if (targets[i] != null)
                {
                    targets[i].Dispose();
                    targets[i] = null;
                }
            }
        }
        
    }
}
